%==================================================================
%  
%  cTraceo - Ray coordinates, Horizontal Array, 2 objects
%  
%  Written by Tordar, Gambelas, Tue Dec  7 14:12:56 WET 2010
%  Based on previous work by Orlando Rodriguez
%  
%==================================================================

addpath('../M-Files/');
addpath('../bin/');
clear all%, close all 

case_title = '''Ray coordinates, Horizontal Array, 2 objects''';
freq       =  100;
Rmaxboxkm  = 1.1; Rmaxbox = Rmaxboxkm*1000;
Rmaxhrykm  = 1.0; Rmaxhry = Rmaxhrykm*1000;
Dmax   =  100;
zs = 50; rs = 500; thetamax = 180;
np2 = 100;
m = 21; ranges = linspace(0,Rmaxhry,m);
n =  1; zr     = 50;

%==================================================================
%  
%  Define source data:
%  
%==================================================================

ray_step = Rmaxbox/1000; 

la = linspace(-thetamax,thetamax,np2);

source_data.ds       = ray_step;
source_data.position = [rs zs];
source_data.rbox     = [-1 Rmaxbox];
source_data.f        = freq;
source_data.thetas   = la;

%==================================================================
%  
%  Define altimetry data:
%  
%==================================================================

%disp('Defining surface characteristics...')

altimetry = [-1 Rmaxbox+1;0 0];

surface_data.type  =   '''V'''; % 
surface_data.ptype =   '''H'''; % Homogeneous
surface_data.units =   '''W'''; % (Attenuation Units) Wavelenght
surface_data.itype =  '''FL'''; % Flat sea surface
surface_data.x     = altimetry; % Surface coordinates
surface_data.properties = [1510.0 600.0 0.99 0.1 0.0];

%==================================================================
%  
%  Define sound speed data:
%  
%==================================================================

%disp('Defining the sound speed profile...')

c0 = 1500;
ssp_data.cdist  = '''c(z,z)'''; % Sound speed profile
ssp_data.cclass = '''ISOV'''; % Isovelocity profile
ssp_data.z      = [0 Dmax]';
ssp_data.r      = [];
ssp_data.c      = [c0 c0]';

%==================================================================
%  
%  Define object data:
%  
%==================================================================

cobjp = 2000.0; cobjs = 200.0; rhoo = 5.0; alphao = 0.0;

npo = 50; 
R0 = 10; factor = 2; ro = [250 750]; zo = [40 60]; 

robj1 =  factor*R0*cos( linspace(pi,0,npo) ) + ro(1);
robj2 =  factor*R0*cos( linspace(pi,0,npo) ) + ro(2);
zup   =         R0*sin( linspace(pi,0,npo) ) + Dmax/2;
zdn   =        -R0*sin( linspace(pi,0,npo) ) + Dmax/2;

xobj1 = [robj1(:)';zdn(:)';zup(:)']; 
xobj2 = [robj2(:)';zdn(:)';zup(:)'];


object_data.nobjects =        2; % Number of objects
object_data.itype    = '''2P'''; % Objects interpolation type
object_data.npobjects(1) =  npo; % Number of points in each object 
object_data.npobjects(2) =  npo; % Number of points in each object
object_data.type(1,:)= '''R''' ; %
object_data.type(2,:)= '''R''' ; %
object_data.units(1,:) ='''W'''; % (Attenuation Units) Wavelenght 
object_data.units(2,:) ='''W'''; % (Attenuation Units) Wavelenght
object_data.properties(1,:) = [cobjp cobjs rhoo alphao alphao];
object_data.properties(2,:) = [cobjp cobjs rhoo alphao alphao];
object_data.x(1,1:3,:)      = xobj1;
object_data.x(2,1:3,:)      = xobj2;

%==================================================================
%  
%  Define bathymetry data:
%  
%==================================================================

%disp('Defining bottom characteristics...')

bathymetry = [-1 Rmaxbox+1;Dmax Dmax];

bottom_data.type   = '''R''' ;
bottom_data.ptype  = '''H''' ; % Homogeneous bottom
bottom_data.units  =  '''W'''; % (Attenuation Units) Wavelenght
bottom_data.itype  = '''FL'''; % Bottom interpolation type 
bottom_data.x      = bathymetry;     % Bottom coordinates 
bottom_data.properties = [1550.0 600.0 2.0 0.1 0.0]; % Bottom properties (speed, speed, density, absorption coefficient)

%==================================================================
%  
%  Define output data:
%  
%==================================================================

%disp('Defining output options...')

depths = zr; 

output_data.ctype       = '''RCO'''; 
output_data.array_shape = '''HRY''';
output_data.r           = ranges;
output_data.z           = depths;
output_data.miss        = 0.5;

%==================================================================
%  
%  Call the function:
%  
%==================================================================

disp('Writing TRACEO waveguide input file...')
wtraceoinfil('flat.in',case_title,source_data,surface_data,ssp_data,object_data,bottom_data,output_data);
clear

tic
disp('Calling sp_cTraceo...')
!../bin/ctraceo_101_sp flat
toc

disp('Reading the output data...')
load rco

%save rays from sp result:
rays_sp = rays;

tic
disp('Calling dp_cTraceo...')
!../bin/ctraceo_101_dp flat
toc

disp('Reading the output data...')
load rco

%save rays from sp result:
rays_dp = rays;

clear rays;

%check that we have the same number of rays:
%length(rays_sp) == length(rays_dp)
nRays = length(rays_sp);

%%
%{
%get the lengths of all rays
dims = zeros(nRays, 3);
for i=1:1:nRays
    dims(i,1:2) = [ length(rays_sp(i).r), length(rays_dp(i).r)];
end

%find least commomn multiplier
for i=1:1:nRays
    dims(i,3) = lcm(dims(i,1), dims(i,2));
end

%interpolate those rays for which the sp and dp versions have generated
%results of differing length.
for i=1:1:nRays
    if dims(i,1) ~= dims(i,2)
        rays_sp(i).r = interp(rays_sp(i).r, dims(i,3)/length(rays_sp(i).r));
        rays_dp(i).r = interp(rays_dp(i).r, dims(i,3)/length(rays_dp(i).r));
    end
end

%calculate mismatch between results
mismatch_r = zeros(length(rays_sp),1);
mismatch_z = zeros(length(rays_sp),1);
for i=1:1:length(rays_sp)
    mismatch_r(i) = norm(rays_sp(i).r - rays_dp(i).r);
    mismatch_z(i) = norm(rays_sp(i).z - rays_dp(i).z);
end
%}
%% NOTE: the above is too computionally expensive

%% compare results
%% method a:
% for all rays which have the same number of coordinates,
% calculate the vector norm of their difference:

meanRelativeError_r = ones(1, nRays)*NaN;
meanRelativeError_z = ones(1, nRays)*NaN;
std_r = ones(1, nRays)*NaN;
std_z = ones(1, nRays)*NaN;
for i = 1:nRays
    if length(rays_sp(i).r) == length(rays_dp(i).r)
        
        % calculate mismatch along r axis
        meanRelativeError_r(i) = mean( ( rays_sp(i).r - rays_dp(i).r)./rays_dp(i).r )*100;
        std_r(i) = std( ( rays_sp(i).r - rays_dp(i).r)./rays_dp(i).r );
        
        
        % Note that because rays which are reflected off the surface have
        % z=0, this will result in NaNs. To avoid this, we need to select
        % only those coordinates which differ from zero:
        ind = find(rays_dp(i).z);
        
        % calculate mismatch along z axis (for all non-zero coordinates)
        meanRelativeError_z(i) = mean( ( rays_sp(i).z(ind) - rays_dp(i).z(ind))./rays_dp(i).z(ind) )*100;
        std_z(i) = std( ( rays_sp(i).z(ind) - rays_dp(i).z(ind))./rays_dp(i).z(ind));
        
    end
end

% plot mismatch curves
% NOTE: this is "mean relative error"
figure
hold on;
plot(meanRelativeError_r, ':r', 'linewidth', 3);
plot(meanRelativeError_z, '-g', 'linewidth', 3);
legend('Along r', 'Along z');
title('Mean relative error along trajectory');
xlabel('Ray index');
yLabel('Mean Relative Error (%)');

%{
% Plot standard deviation
figure
hold on;
plot(std_r, ':r', std_z, '-g');
legend('std_r', 'std_z');
%}

% To make it clearer how [im]precise the SP results are, generate a
% histogram of the mean relative error along ray path:
[hist_r, classes_r] = hist(abs(meanRelativeError_r), round(sqrt(1001)));
[hist_z, classes_z] = hist(abs(meanRelativeError_z), round(sqrt(1001)));

figure
subplot(2,1,1);
bar(classes_r, (hist_r/length(meanRelativeError_r)*100));
title('Mean relative error along range coords of trajectory');
xLabel('Mean Relative Error (%)');
ylabel('Frequency (%)');

subplot(2,1,2);
bar(classes_z, (hist_z/length(meanRelativeError_z)*100));
title('Mean relative error along depth coords of trajectory');
xLabel('Mean Relative Error (%)');
ylabel('Frequency (%)');


%% method b:
% For all rays, get the values of all parameters at the end coordinates,
% and determine the relative error

%create vector of range coordinates
r_sp = zeros(nRays,1);
r_dp = zeros(nRays,1);
z_sp = zeros(nRays,1);
z_dp = zeros(nRays,1);
tau_sp = zeros(nRays,1);
tau_sp = zeros(nRays,1);
amp_sp = zeros(nRays,1);
amp_dp = zeros(nRays,1);
fase_sp = zeros(nRays,1);
fase_dp = zeros(nRays,1);
for i=1:nRays
    r_sp(i) = rays_sp(i).r(end);
    r_dp(i) = rays_dp(i).r(end);
    z_sp(i) = rays_sp(i).z(end);
    z_dp(i) = rays_dp(i).z(end);
    tau_sp(i) = rays_sp(i).tau(end);
    tau_dp(i) = rays_dp(i).tau(end);
    amp_sp(i) = abs(rays_sp(i).amp(end));
    amp_dp(i) = abs(rays_dp(i).amp(end));
    fase_sp(i) = angle(rays_sp(i).z(end));
    fase_dp(i) = angle(rays_dp(i).z(end));
end

%calculate mismatch
meanRelativeError_r = mean( (r_sp - r_dp)./r_dp )
meanRelativeError_z = mean( (z_sp - z_dp)./z_dp )

%{
figure, hold on
plot(rs,zs,'ko',rs,zs,'m*','MarkerSize',16) 
for i = 1:length(rays)
    plot(rays(i).r, rays(i).z)
end
fill(xobj1(1,:),xobj1(2,:),'k')
fill(xobj1(1,:),xobj1(3,:),'k')
fill(xobj2(1,:),xobj2(2,:),'k')
fill(xobj2(1,:),xobj2(3,:),'k')

plot(ranges, depths, 's')
 
grid on, box on
axis([0 Rmaxhry 0 Dmax])
view(0,-90)
hold off 
xlabel('Range (m)')
ylabel('Depth (m)')
title('TRACEO - Isovelocity waveguide, eigenrays')
%}

%%

