%==================================================================
% 
% cTraceo - Munk Profile, Deep Water, All Ray Information
% 
% Written by Tordar,        Faro, Fri Dec 24 02:07:08 WET 2010
% Revised by Emanuel Ey,    30/06/2011
% 
%==================================================================

addpath('../M-Files/');
addpath('../bin/');
clear all%, close all 

disp('Deep water examples:') 
case_title = '''Munk Profile, Deep Water, All Ray Information''';

%==================================================================
% 
% Define source data:
%
%==================================================================

%disp('Defining source characteristics...')

freq   =    50;
Rmaxkm =   101; Rmax = Rmaxkm*1000;
Dmax   =  5000; 

ray_step = Rmax/1000; 

zs = 1000; rs = 0;
np2 = 2;
%la = linspace(-10.2,-9.8,np2);
%la = linspace(-6.2,-5.8,np2);
la = linspace(2.2,1.8,np2);

source_data.ds       = ray_step;
source_data.position = [rs zs];
source_data.rbox     = [rs-1 Rmax];
source_data.f        = freq;
source_data.thetas   = la;

%==================================================================
%
% Define altimetry data:
% 
%==================================================================

%disp('Defining surface characteristics...')


altimetry(1,:) = [rs-2 Rmax+2];
altimetry(2,:) = [0         0];

surface_data.type  =   '''V'''; % 
surface_data.ptype =   '''H'''; % Homogeneous
surface_data.units =   '''W'''; % (Attenuation Units) Wavelenght
surface_data.itype =  '''FL''';
surface_data.x     = altimetry; % Surface coordinates
surface_data.properties = [0 0 0 0 0.0]; % Dummy parameters

%==================================================================
%
% Define sound speed data:
%
%==================================================================

%disp('Defining the sound speed profile...')

c1 = 1500; z1 = 1300;

depths = linspace(0,Dmax,1001);

c = munk( depths, z1, c1 );
 
ssp_data.cdist = '''c(z,z)'''; % Sound speed profile
ssp_data.cclass = '''TABL''';
ssp_data.z    = depths(:);
ssp_data.r    = [];
ssp_data.c    = c(:);

%==================================================================
%  
%  Define object data:
%  
%==================================================================

object_data.nobjects = 0; % No objects

%==================================================================
% 
% Define bathymetry data:
%
%==================================================================

% Gaussian sea mountain: 

bathymetry(1,:) = [rs-2 Rmax+2];
bathymetry(2,:) = [Dmax   Dmax];

bottom_data.type   = '''E''' ;
bottom_data.ptype  = '''H''' ; % Homogeneous bottom
bottom_data.units  =  '''W'''; % (Attenuation Units) Wavelenght
bottom_data.itype  = '''FL'''; % Bottom interpolation type 
bottom_data.x      = bathymetry;     % Bottom coordinates 
bottom_data.properties = [1550.0 600.0 2.0 0.1 0.0]; % Bottom properties (speed, speed, density, absorption coefficient)

%==================================================================
%
% Define output data:
%
%==================================================================

%disp('Defining output options...')

ranges = Rmax; depths = Dmax; 

m = 800;
n = 800;

output_data.ctype       = '''CPR''';
output_data.array_shape = '''RRY''';
output_data.r           = linspace(0,Rmax,m);
output_data.z           = linspace(0,Rmax,n);
output_data.miss        = 0.5;

%==================================================================
%  
%  Call the function:
%  
%==================================================================

disp('Writing TRACEO waveguide input file...')
wtraceoinfil('munk_cpr_a.in',case_title,source_data,surface_data,ssp_data,object_data,bottom_data,output_data);

disp('Calling TRACEO...')
!ctraceo munk_cpr_a

disp('Reading the output data...')
load cpr 
tl = 20.0*log10( abs(p) ); 
tej = flipud( jet( 256 ) );

figure
pcolor(arrayR,arrayZ,tl), shading interp
xlabel('Range (m)')
ylabel('Depth (m)')
        axis([0 Rmax 0 Dmax])
        view(0,-90)

disp('done.')
